(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.globalsRegex = global.globalsRegex || {})));
}(this, (function (exports) { 'use strict';

/**
 * Higher order function which gets configuration for rollup globals
 * and returns function for rollup to properly resolve globals.
 *
 * @example Parameter `config` is object where keys are global which will be set
 * and value is module Id as a string or RegExp that matches it:
 * ```ts
 * {
 *   'ng.core': '@angular/core'
 * }
 * ```
 * This config will set global 'ng.core' for '@angular/core'.
 *
 * @example  If value is RegExp - you can use matched groups in key with '$[indexOfMatch]' sign:
 * ```ts
 * {
 *   'ng.$1': '^@angular/(.*)'
 * }
 * ```
 * This config will set for every '@angular/*' a global 'ng.*'
 * (ex. '@angular/http' => 'ng.http' and so on).
 * NOTE: $0 - group points to whole string same as String.prototype.match result.
 *
 * @example You can also apply transformers on globals ({@see transform()}):
 * ```ts
 * {
 *   '$1': transform('^@angular/(.*)', val => 'ng.' + val)
 * }
 * ```
 * This example will do the same as above one.
 * There are some built-in transformers like {@see fromKebabToCamelCase()} which
 * are often useful with modules like @angular
 * (ex. '@angular/platform-browser' => 'ng.platformBrowser').
 */
function globalsRegex(config) {
    return _globalsRegex.bind(null, config);
}
function _globalsRegex(config, moduleId) {
    var global = '';
    var isGlobalResolved = Object.keys(config)
        .some(function (pattern) {
        var globMatcher = config[pattern];
        if (typeof globMatcher === 'string' && moduleId === globMatcher) {
            global = pattern;
            return true;
        }
        else if (globMatcher instanceof RegExp) {
            var matches = moduleId.match(globMatcher);
            if (matches) {
                global = matches.reduce(function (p, match, i) {
                    if (typeof globMatcher.transform === 'function') {
                        match = globMatcher.transform(match, i);
                    }
                    return p.replace("$" + i, match);
                }, pattern);
                return true;
            }
        }
        return false;
    });
    if (!isGlobalResolved) {
        console.warn("Global not resolved for module '" + moduleId + "'");
    }
    return global;
}

/**
 * Transform kebab-case to camelCase transformer.
 * Use with {@see transform()} function.
 */
var fromKebabToCamelCase = function (val) { return val.replace(/\-(.{1})/g, function (_, w) { return w.toUpperCase(); }); };

/**
 * Apply transformers on result of regex.
 * Transformers can be composed and order of execution is from left to right.
 *
 * @example As simple function:
 * ```ts
 * transformer(/^Hello (.+)/, val => val.toUpperCase())
 * ```
 * Ex: On string 'Hello world' will transform group 'world' => 'WORLD'
 *
 * @example Transformers can be composed:
 * ```ts
 * transformer(/^Hello (.+)/, val => 'beautiful-' + val, fromKebabToCamelCase)
 * ```
 * Ex: On string 'Hello world' will transform group 'world' => 'beautifulWorld'
 * NOTE: Order matters here to get this result
 *
 * @example And you can conditianally apply transformers to some groups by index:
 * ```ts
 * transformer(/^(.+) (.+)/,
 *   (val, i) => i === 0 ? 'Hey! ' + val : val,
 *   (val, i) => i > 0 ? 'beautiful-' + val : val,
 *   fromKebabToCamelCase)
 * ```
 * Ex: On string 'Hello world' will transform group 'Hello' => 'Hey! Hello'
 * and group 'world' => 'beautifulWorld'
 */
function transformer(regex) {
    var transformers = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        transformers[_i - 1] = arguments[_i];
    }
    if (transformers && transformers.length > 0) {
        regex.transform = function (val, i) { return transformers.reduce(function (v, fn) { return fn(v, i); }, val); };
    }
    return regex;
}

/**
 * Preset of useful teplates for globals
 *
 * @example Use in config like this:
 * ```ts
 * globals: {
 *   [GLOBAL.NG2]: GLOBAL.NG2.TPL,
 *   [GLOBAL.RX]: GLOBAL.RX.TPL,
 *   ...
 * }
 * ```
 */
var GLOBAL = {
    NG2: {
        toString: function () {
            return 'ng.$1';
        },
        TPL: transformer(/^\@angular\/(.+)/, fromKebabToCamelCase)
    },
    RX: {
        toString: function () {
            return 'Rx';
        },
        TPL: /^rxjs\/[^/]+$/
    },
    RX_OBSERVABLE: {
        toString: function () {
            return 'Rx.Observable';
        },
        TPL: /^rxjs\/(add\/)?observable\/[^/]+$/
    },
    RX_OPERATOR: {
        toString: function () {
            return 'Rx.Observable.prototype';
        },
        TPL: /^rxjs\/(add\/)?operator\/[^/]+$/
    },
};

exports.globalsRegex = globalsRegex;
exports.transformer = transformer;
exports.fromKebabToCamelCase = fromKebabToCamelCase;
exports.GLOBAL = GLOBAL;

Object.defineProperty(exports, '__esModule', { value: true });

})));
